/*
  *  
  *	see tcpha_be.c for infoes about TCPHA
  *
  */
  
#ifndef _TCPHA_BE_H
#define _TCPHA_BE_H

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/list.h>                 /* for struct list_head */
#include <linux/spinlock.h>             /* for struct rwlock_t */
#include <linux/skbuff.h>               /* for struct sk_buff */
#include <linux/ip.h>                   /* for struct iphdr */
#include <asm/atomic.h>                 /* for struct atomic_t */
#include <linux/sysctl.h>
#include <linux/net.h>
#include <linux/inet.h>
#include <linux/vmalloc.h>
#include <linux/unistd.h>
#include <linux/wait.h>
#include <asm/unistd.h>
#include <linux/list.h>
#include <linux/netfilter.h>
#include <linux/netfilter_ipv4.h>
#include <linux/netfilter_arp.h>
#include <linux/if_ether.h>
#include <linux/if_arp.h>
#include <linux/file.h>
#include <linux/in.h>
#include <linux/random.h>
#include <linux/smp_lock.h>
#include <linux/proc_fs.h>
#include <linux/netdevice.h>		/* for struct neighbour; */
#include <net/dst.h>			/* for struct dst_entry */
#include <net/route.h>			/* for ip_route_output */
#include <net/tcp.h>
#include <net/udp.h>
#include <linux/types.h>
#include <linux/errno.h>          
#include <net/icmp.h>             /* for icmp_send */
#include <net/ip.h>
#include <net/route.h>


/* infoes used by TCP connection Handoff */
struct tcpha_conn_info {
	__u32 			magic;		/* Magic number */
	__u32			daddr;		/* Foreign IPv4 addr			*/
	__u32 			saddr;		/* Sending source			*/
	__u32			rcv_saddr;	/* Bound local IPv4 addr		*/
	__u16			dport;		/* Destination port			*/
	__u16			sport;		/* Source port				*/
	struct tcp_opt	af_tcp;
};

struct tcpha_be_config {
	/* the server IP address and/or port */
	__u32 raddr;            /* real server address */
	__u32 uaddr;		    /* unused address in cluster */
	__u32 vaddr;		    /* shared vitual server address in cluster */
	__u16 port;

	int startservers;
	int maxSpareServers;
	int minSpareServers;

	int maxClients;

	/* address/port to redirect */
	__u32 redirect_addr;
	__u16 redirect_port;
};

struct tcpha_be_service {

	/* server configuration */
	struct tcpha_be_config conf;

	/* server control */
	int start;
	int stop;

	/* run-time variables */
	struct socket *mainsock;
	atomic_t workercount;	/* worker counter */
	atomic_t running;	/* running flag */
};

struct tcpha_be_worker {
	struct tcpha_be_service *svc;	/* service it belongs to */
	int pid;		/* pid of worker */
	volatile __u16 status;	/* worker status */
};

/* a connection to FE */
struct tcpha_be_conn {
	struct socket *fsock;	/* socket connected to FE */

	#ifdef CONFIG_TCPHA_BE_DEBUG
	int magic;
	#endif
};

#define INFO(msg...) printk(KERN_INFO "TCPHA_BE: " msg)
#ifdef CONFIG_TCPHA_BE_DEBUG
#define DBG(msg...) printk(KERN_INFO "TCPHA_BE: " msg)
#define DBG_RL(msg...) 					\
    do {						\
	    if (net_ratelimit())			\
		    printk(KERN_INFO "TCPHA_BE: " msg);	\
    } while (0)
#else
#define DBG(msg...) do {} while (0)
#define DBG_RL(msg...) do {} while (0)
#endif
#define ERR(msg...) printk(KERN_ERR "TCPHA_BE: " msg)
#define ERR_RL(msg...)				\
    do {						\
	    if (net_ratelimit())			\
		    printk(KERN_ERR "TCPHA_BE: " msg);	\
    } while (0)

/* SLAB cache for TCPHA_BE connections */
kmem_cache_t *tcpha_be_conn_cachep;
		
struct tcpha_be_service tbs;

#endif
